import java.util.*;
import javax.persistence.*;
import java.lang.reflect.*;


public class TestClient {
	
  EntityManagerFactory emf = 
         Persistence.createEntityManagerFactory("pu1", new HashMap());
  EntityManager em = emf.createEntityManager();
  public TestClient() {
  }


  public boolean testCreate() {
        
    boolean pass=true;
    boolean s1 = false, s2 = false;
    boolean s3 = false, s4 = false, s5 = false, s6 = false, s7 = false;
    Department dept1 = new Department(1, "Engineering");
    Department dept2 = new Department(2, "Marketing");

    s1 = persistObj(dept1, em);
    s2 = persistObj(dept2, em);

    Employee emp1 = new Employee(1, "Alan", "Frechette", "ext1", dept1);
    EmployeeDel1 emp2 = new EmployeeDel1(2, "Arthur", "Wesley", "ext2", dept1);
    EmployeeDel1 emp3 = new EmployeeDel1(3, "Abe", "White", "ext3", dept2);
    EmployeeDel2 emp4 = new EmployeeDel2(4, "Paul", "Hinz", "ext4", dept2);
    EmployeeDel2 emp5 = new EmployeeDel2(5, "Carla", "Calrson", "ext5", dept2);

    dept1.addEmployee(emp1);
    dept1.addEmployeeDel1(emp2);
    dept2.addEmployeeDel1(emp3);
    dept2.addEmployeeDel2(emp4);
    dept2.addEmployeeDel2(emp5);

    s3 = persistObj(emp1, em);
    s4 = persistObj(emp2, em);
    s5 = persistObj(emp3, em);
    s6 = persistObj(emp4, em);
    s7 = persistObj(emp5, em);
    if (s1 && s2 && s3 && s4 && s5 && s6 && s7){
      pass = true;
    }

    return pass;
  }

  public boolean persistObj(Object v, EntityManager em){
    boolean pass = false;
    EntityTransaction tx = em.getTransaction();  
    try {        
      tx.begin(); 
      em.persist(v);
      tx.commit();
      pass = true;
    } catch(Exception ex){
      ex.printStackTrace();
    }
    return pass;
  }

  public boolean testNativeQuery() {
        
    boolean pass=false;
    EmployeeDel1 emp2 = null;
    EmployeeDel2 emp4 = null;
    String sqlString;
    Query q;
    List result;
    int size;
    int count = 0;
    String check = null;
    EntityTransaction tx = em.getTransaction();  
    try {        

      sqlString="SELECT e.\"JPA_EXT_DEL\" from \"JPA_EMP_DEL\" e";
      q = em.createNativeQuery(sqlString);
      result = q.getResultList();
      size = result.size();
      System.out.println("size of extD1 from JPA_EMP_DEL: " + size);
      for (int i=0; i < size; i++){
	check = (String) result.get(i);
	System.out.println("...check1:" + check);
      }
      if (size == 2 && 
	  (check.equals("ext2") || check.equals("ext3"))) {
	count++;
      }


      sqlString="SELECT e.\"Jpa_Ext_Del\" from \"Jpa_Emp_Del\" e";
      q = em.createNativeQuery(sqlString);
      result = q.getResultList();   
      size = result.size();
      System.out.println("size of extD2 from JPA_Emp_Del: " + size);
      for (int i=0; i < size; i++){
	check = (String) result.get(i);
	System.out.println("...check2: " + check);
      }
      if (size == 2 && 
	  (check.equals("ext4") || check.equals("ext5"))) {
	count++;
      }

      q = em.createNamedQuery("findAllEmployeeDel1s");
      result = q.getResultList();   
      size = result.size();
      System.out.println("size of JPA_EMP_DEL: " + size);
      for (int i=0; i < size; i++){
	check = ((EmployeeDel1) result.get(i)).getExtD1();
	System.out.println("...check3: " + result.get(i));
      }
      if (size == 2 && 
	  (check.equals("ext2") || check.equals("ext3"))) {
	count++;
      }

      q = em.createNamedQuery("findAllEmployeeDel2s");
      result = q.getResultList();   
      size = result.size();
      System.out.println("size of Jpa_Emp_Del: " + size);
      for (int i=0; i < size; i++){
	check = ((EmployeeDel2)result.get(i)).getExtD2();
	System.out.println("...check4: " + result.get(i));
      }
      if (size == 2 && 
	  (check.equals("ext4") || check.equals("ext5"))) {
	count++;
      }

    } catch(Exception ex){
      ex.printStackTrace();
    }

    if (count == 4){
	pass = true;
    }
    return pass;
  }

  public boolean testRead() {
        
    boolean pass=false;
    Set<Employee> e = null;
    Set<EmployeeDel1> eDel1 = null;
    Set<EmployeeDel2> eDel2 = null;
    int num = 0, s = 0;


    Department dept1 = em.find(Department.class, 1);
    System.out.println("dept1="+dept1.toString());
    Department dept2 = em.find(Department.class, 2);
    System.out.println("dept2="+dept2.toString());

    System.out.println("1. init, num="+num + ", s="+s);
    e = dept1.getEmployees();
    s = e.size();
    num += s;
    System.out.println("2. +dept1.e, num="+num + ", s="+s);
    eDel1= dept1.getEmployeeDel1s();
    s = eDel1.size();
    num += s;
    System.out.println("3. +dept1.eDel1, num="+num + ", s="+s);
    eDel2= dept1.getEmployeeDel2s();
    s = eDel2.size();
    num += s;
    System.out.println("4. +dept1.eDel2, num="+num + ", s="+s);
    eDel1 = dept2.getEmployeeDel1s();
    s = eDel1.size();
    num += s;
    System.out.println("5. +dept2.eDel1, num="+num + ", s="+s);
    eDel2 = dept2.getEmployeeDel2s();
    s = eDel2.size();
    num += s;
    for (EmployeeDel2 emp: eDel2) {
        System.out.println(emp.toString());
    }
    System.out.println("6. +dept2.eDel2, num="+num + ", s="+s);
       
    Employee emp1 = em.find(Employee.class, 1);
    System.out.println("emp1="+emp1.toString());

    EmployeeDel1 emp3 = em.find(EmployeeDel1.class, 3);
    System.out.println("emp3="+emp3.toString());
    Department deptForEmp3 = emp3.getDepartment();
    System.out.println("deptForEmp3="+deptForEmp3.toString());

    EmployeeDel2 emp4 = em.find(EmployeeDel2.class, 4);
    System.out.println("emp4="+emp4.toString());
    Department deptForEmp4 = emp4.getDepartment();
    System.out.println("deptForEmp4="+deptForEmp4.toString());

    if (num == 5 && deptForEmp3.equals(deptForEmp4)){
	pass = true;
    }
    return pass;
  }

  public boolean testUpdate() {
        
    boolean pass=false;
    EmployeeDel1 emp2 = null;
    EmployeeDel2 emp4 = null;
    String update1 = "extDel1Update";
    String update2 = "extDel2Update";

    EntityTransaction tx = em.getTransaction();  
    try {        
      tx.begin(); 
      emp2 = em.find(EmployeeDel1.class, 2);
      System.out.println("emp2="+emp2.toString());
      emp2.setExtD1(update1);
      emp4 = em.find(EmployeeDel2.class, 4);
      emp4.setExtD2(update2);
      tx.commit();
    } catch(Exception ex){
      ex.printStackTrace();
    }
    emp2 = em.find(EmployeeDel1.class, 2);
    emp4 = em.find(EmployeeDel2.class, 4);
    String extEmp2 = emp2.getExtD1();
    String extEmp4 = emp4.getExtD2();
    if (extEmp2.equals(update1) && extEmp4.equals(update2)){
	pass = true;
    }
    return pass;
  }


  public boolean testDelete() {
        
    boolean pass=false;
    EmployeeDel1 emp2 = null;
    EmployeeDel2 emp4 = null;

    EntityTransaction tx = em.getTransaction();  
    try {        
      tx.begin(); 
      emp2 = em.find(EmployeeDel1.class, 2);
      Object toDeleteEmp2 = em.merge(emp2);
      em.remove(toDeleteEmp2);
      emp4 = em.find(EmployeeDel2.class, 4);
      Object toDeleteEmp4 = em.merge(emp4);
      em.remove(toDeleteEmp4);
      tx.commit();
    } catch(Exception ex){
      ex.printStackTrace();
    }
    emp2 = em.find(EmployeeDel1.class, 2);
    emp4 = em.find(EmployeeDel2.class, 4);
    if (emp2 == null && emp4 == null){
	pass = true;
    }
    return pass;
  }


  // ******************************************************
  public boolean executeTest(String testname, Object testObj) {
    Method m=null;
    boolean ret=false;  
	
    try {
      m = TestClient.class.getMethod(testname, new Class[] {});
      Object[] arg = {};
      ret = (Boolean) m.invoke(testObj, (Object[])null);     
    } catch (Exception ex){
      ex.printStackTrace();
    }
    return ret;
  } 
    

  public void doClose() {
    em.close();
    emf.close();

  }
  public static void main(String[] args) {    

    String testSuiteID = "test";

    // String testlist[]={"testCreate", 
    String testlist[]={"testCreate", 
		       "testNativeQuery",
                       "testRead", 
                       "testUpdate",
		       "testDelete"};
    if (args.length == 1) {
      testSuiteID=args[0];
    } 

    TestClient tc = new TestClient();

    for (int i=0; i < testlist.length; i++){
      String testname=testlist[i];
      System.out.println("...invoking test" + (i+1) + ": " + testname);
      boolean result = tc.executeTest(testname, tc);
      System.out.println("...finished test"+ (i+1) + ": " + testname+", passed = "+result);
      System.out.println("--------------------------------------");
    }
    tc.doClose();

  }
    
}
